/*
 *  MIDI pattern processor
 *  by Shawn Hargreaves, 1995/96
 *
 *  Mutation routines for inverting and reversing patterns.
 */


#include "defs.h"


MUTATE_FUNC inverter_func, reverser_func;


/* various different types of inversion */

#define INVERT_AROUND_START         1
#define INVERT_USING_SCALE          4


LIST_MEMBER invert_params[] =
{
    /* <description>                <data>                        <coolness> */
    { "Exact Inversion",            INVERT_AROUND_START,          MAX_PROB/4 },
    { "Scalic Inversion",           INVERT_USING_SCALE,           MAX_PROB/4 },
    { NULL, 0, 0 }
};



MUTATOR inverter =
{
    inverter_func,      /* inverts a pattern */
    invert_params       /* what sort of inversion should we use? */
};



MUTATOR reverser =
{
    reverser_func,      /* reverses a pattern */
    NULL                /* don't need an integer parameter */
};



/* inverter_func:
 *  Inverts a pattern.
 */
PATTERN *inverter_func(p1, p2, i)
PATTERN *p1;
PATTERN *p2;
int i;
{
    PATTERN *p;
    int x, c;
    int orig;

    p = clone_pattern(p1);
    if (!p)
	return NULL;

    if (i == INVERT_USING_SCALE) {
	TRACE("inverter_func(INVERT_USING_SCALE)\n");

	/* fill table with the scale used by the pattern */
	fill_temp_with_unique_pitches(p);

	/* for each note in the pattern... */
	for (x=0; x<p->length; x++) {
	    /* find the position in the scale */
	    for (c=0; c<temp_i_size-1; c++)
		if (temp_i[c] == p->note[x].pitch)
		    break;

	    /* and invert it */
	    p->note[x].pitch = temp_i[temp_i_size-c-1];
	}
    }
    else {
	TRACE("inverter_func(INVERT_AROUND_START)\n");
	orig = p->note[0].pitch;

	/* simple inversion about a single note */
	for (x=0; x<p->length; x++)
	    p->note[x].pitch = orig + (orig - p->note[x].pitch);
    }

    return p;
}



/* reverser_func:
 *  Reverses the order of pitches in a pattern (the rhythm is unchanged).
 */
PATTERN *reverser_func(p1, p2, i)
PATTERN *p1;
PATTERN *p2;
int i;
{
    PATTERN *p;
    int x;
    int temp;

    TRACE("reverser_func()\n");

    p = clone_pattern(p1);
    if (!p)
	return NULL;

    for (x=0; x<p->length/2; x++) {
	temp = p->note[x].pitch;
	p->note[x].pitch = p->note[p->length-x-1].pitch;
	p->note[p->length-x-1].pitch = temp;
    }

    return p;
}


